#!/usr/bin/perl -w
use strict;
use Net::IRC;
use Data::Dumper;

my %notices = ('INVITE' => 'INVITE_REJ',
	       'INVITE_AGAIN' => 'INVITE_AGAIN_REJ',
	       'SCORE' => \&cb_score,
	       'ALLSCORES' => \&cb_allscores,
	       'GAMEDONE' => \&cb_gamedone,
	       'GAMEQUIT' => \&cb_gamequit
	      );

my %player_data = ();
my %player_seen = ();

my $host = shift || "localhost";

my $irc = new Net::IRC;
$irc->timeout(180);  # ??????????
my $conn = $irc->newconn(Nick => 'Squaroid',
			 Server => $host);
$conn->add_global_handler('376', \&handle_connect);
$conn->add_global_handler('353', \&handle_namreply);
$irc->start;

sub handle_connect {
  my $conn = shift;
  
  #  print "Joining #squaroid...\n";
  $conn->add_handler('msg', \&handle_msg);
  $conn->add_handler('public', \&handle_public);
  $conn->add_handler('join', \&handle_join);
  $conn->add_handler('notice', \&handle_notice);
  $conn->add_handler('quit', \&handle_part);
  $conn->add_handler('part', \&handle_part);
  $player_data{Squaroid}->{score} = 0;
  $conn->join('#squaroid');
}

sub handle_namreply {
  my $conn = shift;
  my $event = shift;
  
  foreach my $nick (split(/ /,$event->{args}->[3])) {
    next if !$nick || $nick eq "Squaroid";
    $nick =~ s/^[:@\+]//g;
    #    print "Setting $nick\'s scores to 100\n";
    &priv_newplayer($nick);
  }
  &pub_updateplayer($conn, "Squaroid");
}

sub handle_msg {
  my $self = shift;
  my $event = shift;
  my $from = $event->from;

#    my $text = join('\n', $event->args);
#    print <<EOF;
#  >$from<: $text
#  EOF

}

sub handle_public {
  my $self = shift;
  my $event = shift;
  my $from = $event->from;
  my $text = join('\n', $event->args);
print <<EOF;
<$from>: $text
EOF

}

sub handle_join {
  my $self = shift;
  my $event = shift;
  #  print Dumper($event);
  my $text = "Welcome $event->{nick}, your initial score is 100 points.";
  if ($event->nick eq "Squaroid") {
    $player_data{$event->{nick}}->{score} = 0;
  } else {
    $self->privmsg($event->nick,$text);
    &priv_newplayer($event->nick);
  }
  foreach my $nick (keys %player_data) {
    #    print "UPDATE: $nick\n";
    &pub_updateplayer($self, $nick);
  }
}

sub handle_part {
  my $self = shift;
  my $event = shift;
  $player_seen{$event->nick} = time;
  delete $player_data{$event->{nick}};
}

sub handle_notice {
  my $self = shift;
  my $event = shift;
  #  print Dumper($event);
  my ($word) = $event->{args}->[0] =~ /^([^ ]+)/;
  #  print STDERR "ERR: $word $event->{args}->[0]\n";
  if ($word && exists $notices{$word}) {
    if (ref($notices{$word}) eq "CODE") {
      &{$notices{$word}}($self, $event);
      return;
    } else {
      $self->notice($event->{nick}, $notices{$word});
      return undef;
    }
  }
  my $from = $event->from;
  my $text = join('\n', $event->args);
  print <<EOF;
<$from>: $text
EOF
}

#########

sub cb_score {
  my $conn = shift;
  my $event = shift;
  
  my ($word, $nick) = split(/ /, $event->{args}->[0]);
  return if $nick eq $conn->nick;
  my $line = "Nick: <$nick>";
  $line .= " SCORE: ".$player_data{$nick}->{score};
  $line .= " Win: ".$player_data{$nick}->{wins};
  $line .= " Lose: ".$player_data{$nick}->{losses};
  $conn->privmsg($event->nick, $line);
  
}

sub cb_allscores {
  my $conn = shift;
  my $event = shift;
  
  $conn->privmsg($event->nick, "=" x 45);
  foreach my $nick (keys %player_data) {
    next if $nick eq $conn->nick;
    my $line = "Nick: <$nick>";
    $line .= " SCORE: ".$player_data{$nick}->{score};
    $line .= " Win: ".$player_data{$nick}->{wins};
    $line .= " Lose: ".$player_data{$nick}->{losses};
    $conn->privmsg($event->nick, $line);
  }
}

sub cb_gamequit {
  my $conn = shift;
  my $event = shift;
  my $nick = $event->nick;

  $player_data{$nick}->{score} -= 1;
  &pub_updateplayer($conn, $nick);
}

sub cb_gamedone {
  my $conn = shift;
  my $event = shift;
  my $nick = $event->nick;
  my @tokens = split(/ /, $event->{args}->[0]);
  #  my ($word, $opponent, $p1_score, $p2_score, $who) = $event->{args}->[0] =~ /^([a-zA-Z]+]) ([a-zA-Z0-9_]+) (\d+) (\d+) (\d)$/;
  
  my $opponent = $tokens[1];
  my $p1_score = $tokens[2];
  my $p2_score = $tokens[3];
  my $who = $tokens[4];
  
  if (exists $player_data{$opponent}->{gamedone}{$nick}) {
    my ($winscore, $leadscore) = 0;
    my $maxscore = ($player_data{$opponent}->{score}<=$player_data{$nick}->{score}) ? $player_data{$nick}->{score} : $player_data{$opponent}->{score};
    my $minscore = ($player_data{$opponent}->{score}>=$player_data{$nick}->{score}) ? $player_data{$nick}->{score} : $player_data{$opponent}->{score};
    $winscore = $minscore*2 / $maxscore - 1;
    $leadscore = abs($p1_score - $p2_score) / 40 + 1;
    
    # and now: featuring an ugly block!
    my ($n_change, $o_change) = 0;
    if ($p1_score > $p2_score) {
      if ($who == 1) {
	$player_data{$nick}->{wins} += 1;
	$player_data{$opponent}->{losses} += 1;
	$n_change = $winscore + $leadscore;
	$o_change = -1*($winscore + $leadscore);
      } else {
	$player_data{$nick}->{losses} += 1;
	$player_data{$opponent}->{wins} += 1;
	$n_change = -1*($winscore + $leadscore);
	$o_change = $winscore + $leadscore;
      }
    } else {
      if ($who == 2) {
	$player_data{$nick}->{wins} += 1;
	$player_data{$opponent}->{losses} += 1;
	$n_change = $winscore + $leadscore;
	$o_change = -1*($winscore + $leadscore);
      } else {
	$player_data{$nick}->{losses} += 1;
	$player_data{$opponent}->{wins} += 1;
	$n_change = -1*($winscore + $leadscore);
	$o_change = $winscore + $leadscore;
      }
    }
    
    $player_data{$nick}->{score} = sprintf "%.0f", $player_data{$nick}->{score} + $n_change;
    $player_data{$opponent}->{score} = sprintf "%.0f", $player_data{$opponent}->{score} + $o_change;
      
    &pub_updateplayer($conn, $nick);
    &pub_updateplayer($conn, $opponent);
    delete $player_data{$opponent}->{gamedone}{$nick};
    delete $player_data{$opponent}->{gamedone}{$opponent};
    #    print "===============++++++++GO+++++++==============\n";
    #    print Dumper(\%player_data);
    #    print "===============########END######==============\n";
  } else {
    $player_data{$nick}->{gamedone}{$opponent}->[0] = $who;
    $player_data{$nick}->{gamedone}{$opponent}->[1] = $p1_score;
    $player_data{$nick}->{gamedone}{$opponent}->[2] = $p2_score;
    return;
  }

  my $from = $event->from;
  my $text = join('\n', $event->args);
  print <<EOF;
<$from>: $text
EOF
}


############

sub pub_updateplayer {
  my $conn = shift;
  my $player = shift;
  my $score = $player_data{$player}->{score};

  foreach my $nick (keys %player_data) {
    next if $nick eq $conn->nick;
    #    print $nick,":: ","=> UPDATEPLAYER $player $score\n";
    $conn->notice($nick, "UPDATEPLAYER $player $score");
  }
}

############

sub priv_newplayer {
  my $nick = shift;
 
  $player_seen{$nick} = time;
  return if exists $player_data{$nick};
  $player_data{$nick}->{score} = 100;
  $player_data{$nick}->{wins} = 0;
  $player_data{$nick}->{losses} = 0;
}
